/* Program psd.cc       */
/* By Harald Stauss     */
/* Last change 28.02.99 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "tsa.h"
#include "../nrc/nrutil.h" 
#include "../nrc/nrutil.cc"
#include "../nrc/four1.cc"
#include "../nrc/moment.cc"
#include "../nrc/complex.cc"

main(int argc, char *argv[])
{
  short    i;
  int      res;
  unsigned long     fn, n, eospec, rn, nout;
  unsigned long     seg_start, seg_end, nseg, seg_adv;
  float    ave, adev, sdev, var, skew, curt;
  float    *data1, *data2, *data3;
  float    o; /* overlap in % */
  char     *fin,*fout,*ftin,*ftout, *pftin, *pftout;
  REGIST   r,f;
  fcomplex z;


  /* H e l l o   w o r l d */
  cout << "PSD Ver.1.1 by H. Stauss, 1999 \n";

  /* A r r a y s   a l l o c i e r e n */
  fin=(char *)malloc(255);
  fout=(char *)malloc(255);
  ftin=(char *)malloc(255);
  ftout=(char *)malloc(255);
  data1=(float *)malloc(sizeof(float));
  data2=(float *)malloc(sizeof(float));
  data3=(float *)malloc(sizeof(float));

  /* P a r a m e t e r   e i n l e s e n */
  if (argc!=5)
    {
      cout << "Number of parameters wrong! \n";
      cout << "segpsd f_in f_out n o \n";
      cout << "calculates the power spectrum based on segments of n values \n";
      cout << "zero pads to n values if necessary, \n";
      cout << "f_in  : name of input file (*.TSA, *.ASC). \n";
      cout << "f_out : name of output file (*.TSA, *.ASC). \n";
      cout << "n     : number of data values for FFT (determines length of segments) \n";
      cout << "o     : % overlap of segments \n";
      goto ende;
    }
  strcpy(fin,argv[1]);
  strcpy(fout,argv[2]);
  strcpy(ftin,fin);
  strcpy(ftout,fout);
  pftin=strrchr(ftin,'.');
  for (i=0; pftin[i]; i++) pftin[i]=toupper(pftin[i]);
  pftout=strrchr(ftout,'.');
  for (i=0; pftout[i]; i++) pftout[i]=toupper(pftout[i]);
  sscanf(argv[3],"%ul",&fn);
  sscanf(argv[4],"%f",&o);

  printf("Input file  : %s \n",fin);
  printf("Output file : %s \n",fout);
  cout << "FFT on " << fn << " values \n";
  cout << "Overlap " << o << " % \n";

  /* E i n g a b e d a t e i   o e f f n e n */
  if (strcmp(pftin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(pftin,".ASC")==0) r.ReadASCII(fin,1);
  if ((strcmp(pftin,".TSA")!=0) && (strcmp(pftin,".ASC")!=0)) {
    cout << "File type for input not recognized! \n";
    goto ende;
  }
  printf("Number of channels   : %d  \n",r.n);

  data1=(float *)realloc(data1, fn*sizeof(float));
  data2=(float *)realloc(data2, 2*fn*sizeof(float));
  data3=(float *)realloc(data3, (fn+1)*sizeof(float));

  /* M A I N L O O P  */
  rn=r.n; nout=0;
  for (i=0; i<rn; i++) {
    seg_start=0; seg_end=fn-1;
    seg_adv=(unsigned long)((float)fn*(float)(100-o)/(float)100);
    cout << "Channel " << i+1 << ": n = " << r.ch[i].n << "\t";
    if (fn<=r.ch[i].n) {
      nseg=0;
      // Clear data3 array;
      for (n=0; n<=fn; n++) data3[n]=0;
      while (seg_end<r.ch[i].n) {
	// Mittelwert berechnen;
	for (n=seg_start; n<=seg_end; n++) data1[n-seg_start]=r.ch[i].data[n];
	moment(data1-1,fn,&ave,&adev,&sdev,&var,&skew,&curt);
	/* Daten fuer FFT kopieren */
	for (n=0; n<fn; n++) {
	  data2[n*2]=data1[n]-ave;
	  data2[n*2+1]=0;
	}
	four1(data2-1,fn,1);
	// Average spectra in data3 array;
	z.r=data2[0]; z.i=data2[1];
	data3[0]+=Cabs(z)*Cabs(z)/fn/fn;
	for (n=1; n<fn/2; n++) {
	  z.r=data2[n*2]; z.i=data2[n*2+1];
	  data3[n]+=2*Cabs(z)*Cabs(z)/fn/fn;
	} /* endfor */
	z.r=data2[fn];z.i=data2[fn+1];
	data3[fn/2]+=Cabs(z)*Cabs(z)/fn/fn;
	// parameters for next segment;
	seg_start+=seg_adv; seg_end+=seg_adv; nseg++;
      } /* end while */
      // divide data3 by nseg;
      cout << "n_seg = " << nseg << "\n";
      for (n=0; n<fn/2+1; n++) data3[n]=data3[n]/nseg;
      // in neue REGIST copieren;
      f.AddChannel(fn/2+1, r.ch[i].sr, 0, 1, 0, 0,"fft", "fft");
      cout << "do copy\n";
      for (n=0; n<fn/2+1; n++) f.ch[nout].data[n]=data3[n];
      nout++;
      cout << "copy done \n";
    }
    else {
      cout << "too less data values in channel " << i+1 << ". \n";
    }
  } /* endfor */

  /* D a t e i   s c h r e i b e n */
  if (strcmp(pftout,".TSA")==0) f.WriteTSA(fout);
  if (strcmp(pftout,".ASC")==0) f.WriteASCII(fout);
  if ((strcmp(pftout,".TSA")!=0) && (strcmp(pftout,".ASC")!=0)) {
    printf("File type for output not recognized! \n");
    goto ende;
  }

 ende:
  free(data1); free(data2); free(data3);
  free(fin);free(ftin);free(fout);free(ftout);
  return(0);
} /* Ende von main */
